@@ -1,5 +1,5 @@
 package TreePath;
-$TreePath::VERSION = '0.19';
+$TreePath::VERSION = '0.20';
 use utf8;
 use Moose;
 with 'MooseX::Object::Pluggable';
@@ -308,9 +308,9 @@ sub traverse {
   $args ||= {};
   $args->{_count} = 1 if ! defined ($args->{_count});
 
-  my $nofunc = 0;
+  my $hasfunc = 0;
   if ( ! $funcref ) {
-    $nofunc = 1;
+    $hasfunc = 1;
     $funcref = sub {    my ($node, $args) = @_;
                         $args->{_each_nodes} = []
                           if ( ! defined $args->{_each_nodes});
@@ -325,8 +325,10 @@ sub traverse {
     return 0 if ( ! &$funcref( $node, $args ) )
   }
 
+
+
   foreach my $field ( keys %$node) {
-      if ( ref($node->{$field}) eq 'ARRAY' ) {
+      if ( ref($node->{$field}) eq 'ARRAY') {
         foreach my $child ( @{$node->{$field}} ) {
           return 0 if ( ! &$funcref( $child, $args ) );
           $args->{_count}++;
@@ -335,7 +337,7 @@ sub traverse {
       }
   }
 
-  return $args->{_each_nodes} if $nofunc;
+  return $args->{_each_nodes} if $hasfunc;
   return 1;
 }
 
@@ -346,21 +348,14 @@ sub del {
   my @deleted;
   foreach my $node ( @nodes ) {
 
-  my $parent_key = $self->_get_key_name('parent', $node);
-
-      $self->_normalize_node($node);
-
-      my $father = $node->{parent};
-
       $self->_remove_child_father($node);
 
       my $key_children = $self->_key_children($node, $node->{parent});
 
       # traverse child branches and delete it
-      my $nodes = $self->traverse($node);
+      my $Nodes = $self->traverse($node);
 
-      push(@deleted,map { delete $self->tree->{$self->_obj_key($_)} } @$nodes);
-      #$self->_bckd_delete($nodes, "delete " . @$nodes . " node(s)");
+      push(@deleted,map { delete $self->tree->{$self->_obj_key($_)} } @$Nodes);
   }
   return @deleted;
 }
@@ -397,12 +392,6 @@ sub _normalize_node {
   die "key $primary_key is not set in node !"
       if ! defined $node->{$primary_key};
 
-  # check parent key
-  if ( ! $parent ) {
-      my $parent_key =  $self->_get_key_name('parent', $node);
-      die "key $parent_key is not set in node !"
-          if ! defined $node->{$parent_key} && $self->root;
-  }
   my $parent_key = $self->_get_key_name('parent', $node);
 
   # transform obj_id to object reference
@@ -431,7 +420,16 @@ sub _normalize_node {
                   push(@$newarray, $self->tree->{$child});
               }
               elsif ( ref($child) eq 'HASH') {
-                  push(@$newarray, $child);
+                  # push(@$newarray, $child);
+                  my @values = keys %{$child};
+                  # hasref is not already transformed
+                  if ( ! defined $values[1] ) {
+
+                      die "tree->{$values[0]} doesn't exist, can not load " . $self->_obj_key($node) . " !\n"
+                          if ( ! exists $self->tree->{$values[0]} );
+
+                      push(@$newarray, $self->tree->{$values[0]});
+                  }
               }
               else {
                   die "Error : has_many relationship $col (=$child) is not formatted, ex: 'Page_3'\n";
@@ -458,29 +456,27 @@ sub add {
 
   $self->_normalize_node($node, $parent);
 
-  $parent = $node->{$parent_key};
-  if ( ! $parent ){
-      if ( $self->root) {
-          die "root already exist [ node: " .$node->{$self->_get_key_name('source', $node)}. '_' . $node->{$self->_get_key_name('primary', $node)} . " ]!";
+  if ( exists $node->{$parent_key} ){
+      if ( $node->{$parent_key} ) {
+          #print "parent is defined and not null\n";
+          my $key_children = $self->_key_children($node, $node->{$parent_key});
+          push(@{$node->{$parent_key}->{$key_children}}, $node);
       }
-      $self->root($node);
+      else {
+          #print "it's root\n";
+          if ( $self->root) {
+              die "root already exist [ node: " .$node->{$self->_get_key_name('source', $node)}. '_' . $node->{$self->_get_key_name('primary', $node)} . " ]!";
+          }
+          $self->root($node);
+      }
+  }
+  else {
+      #print "parent is not defined, node is an orphan\n";
   }
 
   # save ref node in tree
   $self->tree->{$self->_obj_key($node)} = $node;
 
-  my $key_children;
-  my $update_children = 0;
-
-  # node != root
-  if ( defined $parent && $parent ) {
-      $key_children = $self->_key_children($node, $parent);
-      if ( $parent ) {
-          push(@{$parent->{$key_children}}, $node);
-      }
-  }
-
-
   return $node;
 }
 
@@ -493,8 +489,9 @@ sub update {
   foreach my $k ( sort keys %$datas ) {
 
       if ( ! defined $node->{$k} ){
-              $self->_log("update: can not update, node->{$k} is not defined but _force_create_key is not equal 1");
-              next;
+              #$self->_log("update: can not update, node->{$k} is not defined");
+              #next;
+              $self->_log("node->{$k} is not defined, continue ...");
       }
       elsif ( $node->{$k} eq $datas->{$k} ){next}
 
@@ -594,7 +591,7 @@ This module is at EXPERIMENTAL stage, so use with caution.
                {
                 'id' => '1',
                 'name' => '/',
-                'parent' => '0'
+                'parent' => '0' # or ''
                 'source' => 'Page'
               }
 
