@@ -13,6 +13,15 @@ __PACKAGE__->load_namespaces;
 sub _populate {
   my $self = shift;
 
+  my @files = $self->populate(
+        'File',
+        [
+            [ qw/ id  file              page_id/ ],
+            [     1,  '/tmp/file1.txt', 1        ],
+            [     2,  '/tmp/file2.txt', 1        ],
+        ]
+    );
+
   my @pages = $self->populate(
         'Page',
         [
